{
   X11 Gamma Demo (by Dejan Boras)
   This program is public domain.

   Demonstrates the use of gamma functionality for X11 via the
   XF86VidMode extension.

   Started On:    21.10.2010.
   Last Update:   21.10.2010.
}

{$MODE OBJFPC}{$H+}
PROGRAM x11demo;

USES xf86vmode, x, xlib;

VAR
   {here we'll store the gamma value(factor)}
   gammavalue: single;
   {this is to check errors when converting a string to numerical value}
   code: longint;

   {store the original gamma factors here}
   storedgamma: TXF86VidModeGamma;
   {this contains our new gamma factors}
   gamma: TXF86VidModeGamma;

   {X11 display pointer}
   DPY: PDisplay;
   {X11 screen handle}
   Screen: longint;

   {this is used to get an event from X11}
   event: TXEvent;

{for X11, we need to open a display, which we'll store here}

procedure clamptorange(var val: single; min, max: single);
begin
  if(val < min) then val := min
  else if(val > max) then val := max;
end;

procedure ProcessEvents();
begin
   while (XPending(DPY) > 0) do begin
      XNextEvent(DPY, @event);
   end;
end;

BEGIN
   writeln('Gamma control');

   {get the gamma value from the command line}
   val(ParamStr(1), gammavalue, code);
   if(code = 0) then begin
      writeln('Using gamma value: ', gammavalue:0:2);
      {gamma range is 0.100 to 10.0, so we'll clamp the value to that range}
      clamptorange(gammavalue, 0.100, 10.0);
   end else begin
      if(ParamStr(1) <> '') then begin
         writeln('Invalid gamma value specified: ', ParamStr(1)); exit;
      end;
   end;

   {open the default X11 display}
   DPY := XOpenDisplay(nil);
   if(DPY <> nil) then begin
      writeln('Opened display successfully');
   end else begin
      writeln('Failed to open the display.'); exit;
   end;

   {get default screen for this display}
   Screen := DefaultScreen(DPY);

   {store the desktop gamma values, so we can restore them later}
   writeln('Storing gamma');
   if(XF86VidModeGetGamma(DPY, Screen, @storedgamma) = 0) then
      writeln('Error storing gamma.');

   ProcessEvents();

   {set new gamma values}
   gamma.red := gammavalue;
   gamma.green := gammavalue;
   gamma.blue := gammavalue;

   {set the new gamma}
   writeln('Setting gamma to value: ', gammavalue:0:2);
   if(XF86VidModeSetGamma(DPY, Screen, @gamma) = 0) then
      writeln('Error setting gamma.');

   ProcessEvents();

   {wait a bit for the user}
   writeln('ENTER to exit and restore original gamma.');
   readln();

   {store the desktop gamma values, so we can restore them later}
   writeln('Restoring gamma');
   if(XF86VidModeSetGamma(DPY, Screen, @storedgamma) = 0) then
      writeln('Error restoring gamma.');

   ProcessEvents();

   {close the X11 display}
   writeln('Done with X11.');
   XSetCloseDownMode(DPY, DestroyAll);
   XCloseDisplay(DPY);
END.
